@import url('gbrowse.css');

/* These are frequently-changed options that provide a buttoned-down
   "gray" look */

.datatitle        { background-color: khaki; }
.databody         { background-color: lightgoldenrodyellow; }
.multiplechoicetitle  { background-color: lightblue;
		        font-size: small;
}
.searchtitle      { background-color: #E6EBF0;
	            font-size: small;
		    margin-left: 1em
;		    margin-right: 1em;
}
.searchbody       { background-color: #E6EBF0;
	            font-size: small;
		    margin-left: 1em;
		    margin-right: 1em;
		    margin-bottom: 3px;
}
.settingstitle    { background-color: #E6EBF0;
	            font-size: small;
		    margin-left: 1em;
}
.settingsbody     { background-color: #F2F6FA;
	            font-size: small;
		    margin-left: 1em;
}
.uploadtitle      { background-color: #E6EBF0;
	            font-size: small;
		    margin-left: 1em;
}
.uploadbody       { background-color: #F2F6FA;
	            font-size: small;
		    margin-left: 1em;
}
.tabmenu_active   { cursor: pointer;
		    color: black;
		    background-color: #D0D0D0;
		    font-size: 10pt;
		    font-weight: bold;
                    font-family: sans-serif;
		    padding-top: 5px;
		    padding-left: 5px;
		    padding-right: 5px
	     }
.tabmenu_inactive { cursor: pointer;
		    color: blue;
		    font-size: 10pt;
		    background-color: #F0F0F0;
		    font-weight: bold;
                    font-family: sans-serif;
		    padding-left: 5px;
		    padding-right: 5px
		      }
.tabmenu_inactive:hover {
    text-decoration: underline;
}
.tabbed {
    background-color: transparent;
}
.tabmenu {
    background-color: #F0F0F0;
}
.tabbody {
    background-color: #D0D0D0;
    padding-top: 5px;
}
